@@ -16,7 +16,7 @@ use rr_util::{
     py_types::{assert_tensors_close, ExtraPySelfOps, PyShape, Tensor, PY_UTILS},
     pycall, python_error_exception, simple_from,
     tensor_util::{
-        broadcast_shapes, check_canon_idxs, MiscInputError, Shape, TorchDeviceDtype,
+        broadcast_shapes, check_canon_idxs, MiscInputError, Shape, TorchDevice, TorchDeviceDtype,
         TorchDeviceDtypeOp, TorchDtype,
     },
 };
@@ -494,7 +494,7 @@ impl SpecTrait for GeneralFunctionSetDDSpec {
 
     fn name(&self) -> &'static str {
         Name::new(&format!(
-            "cast_{:?}_{:?}",
+            "cast_from_{:?}_to_{:?}",
             self.input_required_compatibility, self.output
         ))
         .into()
@@ -1048,9 +1048,12 @@ impl GeneralFunction {
         name: Option<Name>,
     ) -> Result<Option<Self>> {
         // parse index case
-        static RE: Lazy<Regex> =
+        static RE_GEN_INDEX: Lazy<Regex> =
             Lazy::new(|| Regex::new(r"^gen_index_at_(-?\d+)(_batch_x)?(_c)?$").unwrap());
-        let spec = if let Some(re_captures) = RE.captures(&spec_name) {
+        static RE_CAST: Lazy<Regex> = Lazy::new(|| {
+            Regex::new(r"^cast_from_\{device:([a-zA-Z0-9]+),dtype:([a-zA-Z0-9]+)\}_to_\{device:([a-zA-Z0-9]+),dtype:([a-zA-Z0-9]+)\}$").unwrap()
+        });
+        let spec = if let Some(re_captures) = RE_GEN_INDEX.captures(&spec_name) {
             let index_dim = re_captures
                 .get(1)
                 .unwrap()
@@ -1065,6 +1068,38 @@ impl GeneralFunction {
                 check_index_ints,
             }
             .into()
+        } else if let Some(re_captures) = RE_CAST.captures(&spec_name) {
+            let device_in = re_captures.get(1).unwrap().as_str();
+            let dtype_in = re_captures.get(2).unwrap().as_str();
+            let device_out = re_captures.get(3).unwrap().as_str();
+            let dtype_out = re_captures.get(4).unwrap().as_str();
+            GeneralFunctionSetDDSpec {
+                output: TorchDeviceDtypeOp {
+                    device: if device_out == "None" {
+                        None
+                    } else {
+                        Some(device_out.to_owned().try_into().unwrap())
+                    },
+                    dtype: if dtype_out == "None" {
+                        None
+                    } else {
+                        Some(dtype_out.to_owned().try_into().unwrap())
+                    },
+                },
+                input_required_compatibility: TorchDeviceDtypeOp {
+                    device: if device_in == "None" {
+                        None
+                    } else {
+                        Some(device_in.to_owned().try_into().unwrap())
+                    },
+                    dtype: if dtype_in == "None" {
+                        None
+                    } else {
+                        Some(dtype_in.to_owned().try_into().unwrap())
+                    },
+                },
+            }
+            .into()
         } else if spec_name.str() == "pow" {
             GeneralFunctionPowSpec {}.into()
         } else if let Some(spec) = SPECS.get(&spec_name) {
