@@ -326,14 +326,14 @@ pub fn einsum_nest_path(einsum: &Einsum, path: Vec<Vec<usize>>) -> Einsum {
 
 #[pyfunction]
 #[pyo3(name = "einsum_nest_optimize")]
-pub fn einsum_nest_optimize_py(einsum: &Einsum) -> Option<Einsum> {
+pub fn einsum_nest_optimize_py(einsum: &Einsum) -> Result<Einsum> {
     einsum_nest_optimize(
         einsum,
         &mut OptimizationContext::new_settings(Default::default()),
     )
 }
 
-pub fn einsum_nest_optimize(einsum: &Einsum, context: &mut OptimizationContext) -> Option<Einsum> {
+pub fn einsum_nest_optimize(einsum: &Einsum, context: &mut OptimizationContext) -> Result<Einsum> {
     // by default dont pre-contract subsets of arrayconstants
     let path = optimize_einsum_spec_cached(
         einsum.get_spec(),
@@ -345,7 +345,7 @@ pub fn einsum_nest_optimize(einsum: &Einsum, context: &mut OptimizationContext)
         })),
     )?;
     // let path = einsum.get_spec().optimize_dp(None,None,None);
-    Some(einsum_nest_path(einsum, path))
+    Ok(einsum_nest_path(einsum, path))
 }
 
 /// corresponds to python function `construct_diag`
