@@ -18,7 +18,7 @@ pub fn strip_names_and_tags(circuit: CircuitRc, context: &mut OptimizationContex
             if let Some(tag) = circuit.as_tag() {
                 return Some(tag.node.clone());
             }
-            if circuit.name().is_some() {
+            if circuit.info().name.is_some() {
                 if circuit.is_irreducible_node() {
                     return Some(Tag::nrc(circuit, Uuid::nil(), None).rename(None));
                 } else {
