@@ -54,6 +54,59 @@ impl IntoPy<PyObject> for Instruction {
     }
 }
 
+/// allow for having tensors directly so we can bypass hashing (which isn't
+/// important for scheduling)
+#[derive(Clone, Debug, FromPyObject)]
+pub enum ScheduleConstant {
+    Circ(IrreducibleNode),
+    Tensor(Tensor),
+}
+
+impl IntoPy<PyObject> for ScheduleConstant {
+    fn into_py(self, py: Python<'_>) -> PyObject {
+        match self {
+            ScheduleConstant::Circ(circ) => circ.into_py(py),
+            ScheduleConstant::Tensor(tensor) => tensor.into_py(py),
+        }
+    }
+}
+
+impl ScheduleConstant {
+    pub fn as_symbol(&self) -> Option<&Symbol> {
+        if let Self::Circ(circ) = self {
+            circ.as_symbol()
+        } else {
+            None
+        }
+    }
+    pub fn as_array(&self) -> Option<&Array> {
+        if let Self::Circ(circ) = self {
+            circ.as_array()
+        } else {
+            None
+        }
+    }
+    pub fn as_tensor(&self) -> Option<&Tensor> {
+        if let Self::Tensor(t) = self {
+            Some(t)
+        } else {
+            None
+        }
+    }
+    pub fn convert_as_tensor(&self) -> Option<&Tensor> {
+        match self {
+            Self::Circ(circ) => circ.as_array().map(|x| &x.value),
+            Self::Tensor(tensor) => Some(tensor),
+        }
+    }
+    pub fn shape(&self) -> &Shape {
+        match self {
+            Self::Circ(circ) => &circ.info().shape,
+            Self::Tensor(tensor) => tensor.shape(),
+        }
+    }
+}
+
 /// Schedule is an optimized sequence of operations to perform to compute a circuit
 /// it keeps arrayconstants seperately, and replacing arrayconstants in a schedule should produce the same
 /// output + ~performance as optimizing the circuit with different arrayconstants from the beginning
@@ -65,7 +118,7 @@ pub struct Schedule {
     #[pyo3(get)]
     pub instructions: Vec<Instruction>,
     #[pyo3(get)]
-    pub constants: HashMap<usize, IrreducibleNode>,
+    pub constants: HashMap<usize, ScheduleConstant>,
     // keep scalar constants seperate so adjust numerical scale can work without losing precision
     // before when these were in tensors, they had to be right dtype and therefore overflowed when
     // adjustment needed
@@ -149,7 +202,7 @@ impl Schedule {
                 bail!("tensor shape doesn't match entry shape!");
             }
 
-            *entry.into_mut() = Array::new(v, None).into();
+            *entry.into_mut() = ScheduleConstant::Tensor(v);
         }
         Ok(result)
     }
@@ -162,7 +215,7 @@ impl Schedule {
             let maybe_tensor: Option<Tensor> =
                 Python::with_gil(|py| pycall!(f, (PyBytes::new(py, key),)));
             if let Some(t) = maybe_tensor {
-                result.constants.insert(*id, Array::new(t, None).into());
+                result.constants.insert(*id, ScheduleConstant::Tensor(t));
             }
         });
         Ok(result)
@@ -209,6 +262,17 @@ impl Schedule {
         }
         Ok(())
     }
+    pub fn check_no_raw_tensor_constants(&self) -> Result<()> {
+        let mut iter = self
+            .constants
+            .iter()
+            .filter_map(|x| x.1.as_tensor())
+            .peekable();
+        if iter.peek().is_some() {
+            bail!("has raw tensor constant")
+        }
+        Ok(())
+    }
 
     #[args(settings = "Default::default()")]
     pub fn evaluate_many(&self, settings: OptimizationSettings) -> Result<Vec<Tensor>> {
@@ -246,7 +310,7 @@ impl Schedule {
             constant_mem: self
                 .constants
                 .iter()
-                .map(|(_h, t)| t.info().shape.iter().product::<usize>())
+                .map(|(_h, t)| t.shape().iter().product::<usize>())
                 .sum(),
             max_circuit_set: biggest.values().cloned().collect(),
         }
@@ -315,7 +379,7 @@ impl Display for Schedule {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         write!(
             f,
-            "Schedule: instructions\n{}\nArrays: {} Symbols: {} Scalars: {}",
+            "Schedule: instructions\n{}\nTensors: {} Arrays: {} Symbols: {} Scalars: {}",
             self.instructions
                 .iter()
                 .filter_map(|i| {
@@ -335,6 +399,11 @@ impl Display for Schedule {
                 })
                 .collect::<Vec<String>>()
                 .join("\n"),
+            self.constants
+                .iter()
+                .filter_map(|(k, c)| c.as_tensor().map(|_| k.to_string()))
+                .collect::<Vec<String>>()
+                .join(" "),
             self.constants
                 .iter()
                 .filter_map(|(k, c)| c.as_array().map(|_| k.to_string()))
@@ -471,7 +540,7 @@ pub fn evaluate_schedule(schedule: &Schedule) -> Result<HashMap<usize, Tensor>>
     let mut live: HashMap<usize, Tensor> = schedule
         .constants
         .iter()
-        .map(|x| x.1.as_array().map(|z| (*x.0, z.value.clone())))
+        .map(|x| x.1.convert_as_tensor().map(|z| (*x.0, z.clone())))
         .collect::<Option<HashMap<_, _>>>()
         .unwrap();
     live.extend(
@@ -563,7 +632,7 @@ pub fn evaluate_schedule_adjust_numerical_scale(
     let mut live: HashMap<usize, (Tensor, f64)> = schedule
         .constants
         .iter()
-        .map(|(key, x)| (*key, clamp(&(x.as_array().unwrap().value.clone(), 1.0))))
+        .map(|(key, x)| (*key, clamp(&(x.convert_as_tensor().unwrap().clone(), 1.0))))
         .collect();
     live.extend(schedule.scalars.iter().map(|(h, s)| {
         (*h, {
@@ -684,7 +753,7 @@ pub fn order_to_schedule(
     device_dtype: TorchDeviceDtype,
 ) -> (Schedule, Vec<usize>) {
     let mut circ_to_id: HashMap<HashBytes, usize> = Default::default();
-    let constants: HashMap<usize, IrreducibleNode> = constants
+    let constants: HashMap<usize, _> = constants
         .iter()
         .map(|x| {
             circ_to_id.insert(x.info().hash, circ_to_id.len());
@@ -693,7 +762,6 @@ pub fn order_to_schedule(
         .collect();
     let mut result: Schedule = Schedule {
         instructions: vec![],
-        constants: constants.clone(),
         scalars: scalars
             .iter()
             .map(|x| {
@@ -708,6 +776,10 @@ pub fn order_to_schedule(
             .iter()
             .map(|(id, node)| (node.info().hash, *id))
             .collect(),
+        constants: constants
+            .into_iter()
+            .map(|(h, n)| (h, ScheduleConstant::Circ(n)))
+            .collect(),
     };
 
     let mut seen_dependencies: HashSet<usize> = HashSet::default();
@@ -994,6 +1066,7 @@ fn schedule_module_spec(spec: &ModuleSpec, context: &mut OptimizationContext) ->
         })
 }
 
+/// note: this doesn't allow for any tensors in constants
 fn replace_module_schedules(schedule: &Schedule, context: &mut OptimizationContext) -> Schedule {
     let mut result = schedule.clone();
     let mut scalar_to_id: HashMap<Scalar, usize> = schedule
@@ -1004,7 +1077,7 @@ fn replace_module_schedules(schedule: &Schedule, context: &mut OptimizationConte
     let mut irreducible_to_id: HashMap<IrreducibleNode, usize> = schedule
         .constants
         .iter()
-        .map(|(a, b)| (b.clone(), *a))
+        .map(|(a, b)| (unwrap!(b, ScheduleConstant::Circ).clone(), *a))
         .collect();
     result.instructions = vec![];
     let mut next_key = schedule.next_key();
@@ -1033,6 +1106,7 @@ fn replace_module_schedules(schedule: &Schedule, context: &mut OptimizationConte
                         inner_to_outer_key.try_insert(*inner_k, outer).unwrap();
                     }
                     for (inner_k, irreducible) in &inner_schedule.constants {
+                        let irreducible = unwrap!(irreducible, ScheduleConstant::Circ);
                         let outer =
                             irreducible_to_id
                                 .get(irreducible)
@@ -1047,7 +1121,10 @@ fn replace_module_schedules(schedule: &Schedule, context: &mut OptimizationConte
                                     } else {
                                         let this_key = next_key;
                                         irreducible_to_id.insert(irreducible.clone(), this_key);
-                                        result.constants.insert(this_key, irreducible.clone());
+                                        result.constants.insert(
+                                            this_key,
+                                            ScheduleConstant::Circ(irreducible.clone()),
+                                        );
                                         next_key += 1;
                                         this_key
                                     }
