@@ -15,6 +15,7 @@ use regex::{Captures, Regex};
 use rr_util::{
     eq_by_big_hash::EqByBigHash,
     make_single_many,
+    name::Name,
     py_types::PyCallable,
     setup_callable, simple_default, simple_from,
     util::{arc_unwrap_or_clone, EmptySingleMany as ESM},
@@ -26,12 +27,13 @@ use crate::{BoundAnyFound, IterativeMatcher, IterativeMatcherRc};
 
 make_single_many!(TypeTags, CircuitType, BTreeSet);
 make_single_many!(Strings, String, BTreeSet);
+make_single_many!(Names, Name, BTreeSet);
 make_single_many!(Circuits, CircuitRc, BTreeSet);
 
 #[derive(Clone, FromPyObject)]
 pub enum MatcherFromPyBase {
     Always(bool),
-    Name(Strings),
+    Name(Names),
     Type(TypeTags),
     Regex(RegexWrap),
     EqM(Circuits),
@@ -49,7 +51,7 @@ pub enum MatcherFromPy {
 #[derive(Clone, Debug)]
 pub enum MatcherData {
     Always(bool),
-    Name(BTreeSet<String>),
+    Name(BTreeSet<Name>),
     Type(BTreeSet<CircuitType>),
     Regex(RegexWrap),
     EqM(BTreeSet<CircuitRc>),
@@ -277,10 +279,7 @@ impl Matcher {
 
         match &self.data {
             MatcherData::Name(names) => {
-                let found_names = matched
-                    .into_iter()
-                    .filter_map(|x| x.name().map(|s| s.to_owned()))
-                    .collect();
+                let found_names = matched.into_iter().filter_map(|x| x.info().name).collect();
                 run_on_set(names, &found_names, "names", "name", |x| format!("{:?}", x))
             }
             MatcherData::Type(types) => {
@@ -295,7 +294,7 @@ impl Matcher {
                 |x| -> String {
                     "[".to_owned()
                         + &x.into_iter()
-                            .map(|x| format!("{}({:?})", x.variant_string(), x.name()))
+                            .map(|x| format!("{}({:?})", x.variant_string(), x.info().name))
                             .collect::<Vec<_>>()
                             .join(", ")
                         + "]"
@@ -346,9 +345,17 @@ impl Matcher {
     pub fn call(&self, circuit: CircuitRc) -> Result<bool> {
         let ret = match &self.data {
             &MatcherData::Always(x) => x,
-            MatcherData::Name(names) => circuit.name().map(|x| names.contains(x)).unwrap_or(false),
+            MatcherData::Name(names) => circuit
+                .info()
+                .name
+                .map(|x| names.contains(&x))
+                .unwrap_or(false),
             MatcherData::Type(type_tags) => type_tags.contains(&circuit.type_tag()),
-            MatcherData::Regex(r) => circuit.name().map(|x| r.call(x)).unwrap_or(false),
+            MatcherData::Regex(r) => circuit
+                .info()
+                .name
+                .map(|x| r.call(x.str()))
+                .unwrap_or(false),
             MatcherData::EqM(circs) => circs.contains(&circuit),
             MatcherData::AnyFound(any_found) => any_found.lock().unwrap().are_any_found(circuit)?,
             MatcherData::Raw(f) => f.0(circuit)?,
