@@ -443,29 +443,30 @@ impl Tensor {
         })
     }
 
-    pub fn hashed(&self) -> Tensor {
+    pub fn hashed(&self) -> Result<Tensor> {
         if self.hash.is_some() {
-            self.clone()
+            Ok(self.clone())
         } else {
-            Self {
+            Ok(Self {
                 tensor: self.tensor.clone(),
                 shape: self.shape.clone(),
                 hash: Python::with_gil(|py| {
-                    HASH_TENSOR
-                        .0
-                        .call(py, (self.tensor.clone(),), None)
-                        .expect("Perhaps tensor type wasn't handled in `dtypes_to_bytes`? (should be added...)")
-                        .extract(py)
-                        .unwrap()
-                }),
-            }
+                    Ok::<Option<HashBytes>, anyhow::Error>(
+                        HASH_TENSOR
+                            .0
+                            .call(py, (self.tensor.clone(),), None)?
+                            .extract(py)
+                            .unwrap(),
+                    )
+                })?,
+            })
         }
     }
 }
 
 #[pyfunction]
-pub fn hash_tensor(t: Tensor) -> HashBytesToPy {
-    (*t.hashed().hash().unwrap()).into()
+pub fn hash_tensor(t: Tensor) -> Result<HashBytesToPy> {
+    Ok((*t.hashed()?.hash().unwrap()).into())
 }
 
 #[derive(FromPyObject)]
