@@ -8,6 +8,8 @@ use mimalloc::MiMalloc;
 static GLOBAL: MiMalloc = MiMalloc;
 
 use pyo3::{types::PyModule, wrap_pyfunction, PyResult, Python};
+
+use crate::error::ExceptionWithRustContext;
 /// A Python module implemented in Rust. The name of this function must match
 /// the `lib.name` setting in the `Cargo.toml`, else Python will not be able to
 /// import the module.
@@ -583,6 +585,8 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
         m
     )?)?;
 
+    m.add_class::<ExceptionWithRustContext>()?;
+
     error::register_exceptions(py, m)?;
 
     Ok(())
